//Title:        Enum (interface)
//Version:      0.1
//Copyright:    Copyright (c) 1999
//Author:       Elwood Wray Johnson
//Organization: Object Data Management Group (ODMG)
//Description:  Object Definition Language (ODL)

package org.odmg.odl.metaobjects;

import java.util.*;

public abstract class Enum
{
  private static Hashtable nextOrdinal = new Hashtable();
  private static Hashtable size = new Hashtable();
  private static Hashtable first = new Hashtable();
  private static Hashtable last = new Hashtable();
  private transient Enum prev = null;
  private transient Enum next = null;

  public Enum(int ordinal)
  {
    init(ordinal);
  }
  public Enum()
  {
    init(getNextOrdinal(this));
  }

  private static int getNextOrdinal(final Enum ofThis)
  {
    return nextOrdinal.get(ofThis.getClass()) == null? 0 :
      ((Integer)nextOrdinal.get(ofThis.getClass())).intValue();
  }

  private static void setNextOrdinal(final Enum ofThis, int newNextOrdinal)
  {
    nextOrdinal.put(ofThis.getClass(), new Integer(newNextOrdinal));
  }

  private static int getSize(final Enum ofThis)
  {
    return size.get(ofThis.getClass()) == null? 0 :
      ((Integer)size.get(ofThis.getClass())).intValue();
  }

  private static void setSize(final Enum ofThis, int newSize)
  {
    size.put(ofThis.getClass(), new Integer(newSize));
  }

  private static Enum getFirst(final Enum ofThis)
  {
    return first.get(ofThis.getClass()) == null? null :
      (Enum)first.get(ofThis.getClass());
  }

  private static void setFirst(final Enum newFirst)
  {
    first.put(newFirst.getClass(), newFirst);
  }

  private static Enum getLast(final Enum ofThis)
  {
    return last.get(ofThis.getClass()) == null? null :(Enum)last.get(ofThis.getClass());
  }

  private static void setLast(final Enum newLast)
  {
    last.put(newLast.getClass(), newLast);
  }

  private void init(int ordinal)
  {
    setSize(this, getSize(this) + 1);
    this.ordinal = ordinal;
    this.setNextOrdinal(this, ordinal + 1);
    if (this.getFirst(this) == null)
      this.setFirst(this);
    if (this.getLast(this) != null)
    {
      this.prev = this.getLast(this);
      this.getLast(this).next = this;
    }
    this.setLast(this);
  }

  static public java.util.Enumeration elements(final Enum ofThis)
  {
    return new java.util.Enumeration()
    {
      private Enum curr = getFirst(ofThis);
      public boolean hasMoreElements()
      {
        return curr != null;
      }
      public Object nextElement()
      {
        Enum c = curr;
        curr = curr.next();
        return c;
      }
    };
  }

  public String toString() { return Integer.toString(this.getOrdinal()); }
  public Enum prev() { return this.prev; }
  public Enum next() { return this.next; }

  private transient int ordinal = 0;
  public void setOrdinal(int newOrdinal)
  {
    ordinal = newOrdinal;
  }
  public int getOrdinal()
  {
    return ordinal;
  }
}

