//Title:        Interface (interface)
//Version:      0.1
//Copyright:    Copyright (c) 1999
//Author:       Elwood Wray Johnson
//Organization: Object Data Management Group (ODMG)
//Description:  Object Definition Language (ODL)

package org.odmg.odl.metaobjects;

import java.beans.*;
import org.odmg.oml.*;

public interface Interface extends Type, DefiningScope
{
  interface ParameterSpec
  {
    public String getParamName();
    public void setParamName(String newParamName);

    public void setParamMode(Direction newParamMode);
    public Direction getParamMode();

    public void setParamType(Type newParamType);
    public Type getParamType();
  }

  public DSet getInherits();
  public void setInherits(DSet newInherits);
  static final public FeatureDescriptor inheritsInverse =
    Relationship.Inverse.getDescriptor(Inheritance.class, "derivesFrom");

  public DSet getDerives();
  public void setDerives(DSet newDerives);
  static final public FeatureDescriptor derivesInverse =
    Relationship.Inverse.getDescriptor(Inheritance.class, "inheritsTo");

  public final class BadParameter extends java.lang.Exception
  {
    public BadParameter(String reason)
    {
    	super(reason);
    }
  }

  public final class BadRelationship extends java.lang.Exception
  {
    public BadRelationship(String reason)
    {
    	super(reason);
    }
  }

  public Attribute addAttribute(String attrName, Type attrType)
    throws Scope.DuplicateName;
  public Relationship addRelationship(String relname, Type relType,
    Relationship relTraversal) throws Scope.DuplicateName, BadRelationship;
  public Operation addOperation(String opName, Type opResult,
    ParameterSpec opParams[]) throws Scope.DuplicateName, BadParameter;
  public void removeAttribute(Attribute object) throws CannotRemove;
  public void removeRelationship(Relationship object) throws CannotRemove;
  public void removeOperation(Operation object) throws CannotRemove;
}

