//Title:        PrimitiveKind (interface)
//Version:      0.1
//Copyright:    Copyright (c) 1999
//Author:       Elwood Wray Johnson
//Organization: Object Data Management Group (ODMG)
//Description:  Object Definition Language (ODL)

package org.odmg.odl.metaobjects;

public final class PrimitiveKind extends Enum
{
  static final public PrimitiveKind pkBoolean  = new PrimitiveKind("Boolean", java.lang.Boolean.TYPE, java.lang.Boolean.class);
  static final public PrimitiveKind pkChar     = new PrimitiveKind("Char", java.lang.Character.TYPE, java.lang.Character.class);
  static final public PrimitiveKind pkShort    = new PrimitiveKind("Short", java.lang.Short.TYPE, java.lang.Short.class);
  static final public PrimitiveKind pkUshort   = new PrimitiveKind("Ushort", java.lang.Integer.TYPE, java.lang.Integer.class);
  static final public PrimitiveKind pkLong     = new PrimitiveKind("Long", java.lang.Integer.TYPE, java.lang.Integer.class);
  static final public PrimitiveKind pkUlong    = new PrimitiveKind("Ulong", java.lang.Long.TYPE, java.lang.Long.class);
  static final public PrimitiveKind pkFloat    = new PrimitiveKind("Float", java.lang.Float.TYPE, java.lang.Float.class);
  static final public PrimitiveKind pkDouble   = new PrimitiveKind("Double", java.lang.Double.TYPE, java.lang.Double.class);
  static final public PrimitiveKind pkOctet    = new PrimitiveKind("Octet", java.lang.Byte.TYPE, java.lang.Byte.class);
  static final public PrimitiveKind pkString   = new PrimitiveKind("String", java.lang.String.class);
  static final public PrimitiveKind pkVoid     = new PrimitiveKind("Void", java.lang.Void.TYPE, java.lang.Void.class);
  static final public PrimitiveKind pkAny      = new PrimitiveKind("Any", java.lang.Object.class);
  static final public PrimitiveKind pkDate     = new PrimitiveKind("Date", java.sql.Date.class);
  static final public PrimitiveKind pkTime     = new PrimitiveKind("Time", java.sql.Time.class);
  static final public PrimitiveKind pkTimstamp = new PrimitiveKind("Timestamp", java.sql.Timestamp.class);

  private PrimitiveKind(String name, java.lang.Class newPrimitiveType)
  {
    setName(name);
    setPrimitiveType(newPrimitiveType);
    setPrimitiveClass(null);
  }

  private PrimitiveKind(String name, java.lang.Class newPrimitiveType,
    java.lang.Class newPrimitiveClass)
  {
    this(name, newPrimitiveType);
    setPrimitiveClass(newPrimitiveClass);
  }

  static public java.util.Enumeration elements()
  {
    return Enum.elements(pkBoolean);
  }

  private java.lang.Class primitiveClass;
  public void setPrimitiveClass(java.lang.Class newPrimitiveClass)
  {
    primitiveClass = newPrimitiveClass;
  }
  public java.lang.Class getPrimitiveClass()
  {
    return primitiveType;
  }

  private String name;
  public void setName(String newName)
  {
    name = newName;
  }
  public String getName()
  {
    return name;
  }

  private java.lang.Class primitiveType;
  public void setPrimitiveType(java.lang.Class newPrimitiveType)
  {
    primitiveType = newPrimitiveType;
  }
  public java.lang.Class getPrimitiveType()
  {
    return primitiveType;
  }

  static public boolean isPrimitive(java.lang.Class theClass)
  {
    for (java.util.Enumeration e = elements(); e.hasMoreElements(); )
    {
      PrimitiveKind thisPrimitive = (PrimitiveKind)e.nextElement();
      if (theClass == thisPrimitive.getPrimitiveType() ||
        theClass == thisPrimitive.getPrimitiveClass())
      {
        return true;
      }
    }
    return false;
  }
}
